from django.contrib.auth import get_user_model
from django.core.files.base import ContentFile
from django.test import override_settings
from rest_framework import status
from rest_framework.test import APITestCase, APIClient
from djangorestfilemanager.models import File
from django.contrib.auth.models import Permission, ContentType


@override_settings(ROOT_URLCONF='djangorestfilemanager.urls', PUB_SUB_EMIT=False)
class TestUploadFile(APITestCase):

    def setUp(self):
        User = get_user_model()
        self.content_type = ContentType.objects.create(app_label='file', model='file')
        self.perm = Permission.objects.create(content_type=self.content_type, codename='download_file', name='Download File')
        self.superuser = User.objects.create(username='askywalker', is_superuser=True)
        self.owner = User.objects.create(username='lskywalker')
        self.not_owner = User.objects.create(username='chewe')
        self.not_owner.user_permissions.add(self.perm)
        file = ContentFile(b'content', name='plain.txt')
        self.file = File.objects.create(username=self.owner.username, file=file, name=file.name, share=False)
        self.client = APIClient()

    def tearDown(self):
        for file in File.objects.all():
            file.file.delete()

    def test_unauthenticated_user_cannot_download_file(self):
        response = self.client.get('/v1/files/{}/'.format(self.file.uuid))
        self.assertEqual(status.HTTP_403_FORBIDDEN, response.status_code)

    def _set_file_as_not_shared(self):
        self.file.share = False
        self.file.permission = ''
        self.file.save()

    def _set_file_as_shared_with_fake_permission(self):
        self.file.share = True
        self.file.permission = 'fake permission'
        self.file.save()

    def _set_file_as_shared_with_right_permission(self):
        self.file.share = True
        self.file.permission = 'file.download_file'
        self.file.save()

    def _set_file_as_shared_with_empty_permission(self):
        self.file.share = True
        self.file.permission = ''
        self.file.save()

    def test_only_owner_can_download_not_shared_file(self):
        self._set_file_as_not_shared()
        self.client.force_authenticate(self.owner)
        response = self.client.get('/v1/files/{}/'.format(self.file.uuid))
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.client.force_authenticate(self.not_owner)
        response = self.client.get('/v1/files/{}/'.format(self.file.uuid))
        self.assertEqual(status.HTTP_403_FORBIDDEN, response.status_code)

    def test_superuser_can_download_not_shared_file(self):
        self._set_file_as_not_shared()
        self.client.force_authenticate(self.superuser)
        response = self.client.get('/v1/files/{}/'.format(self.file.uuid))
        self.assertEqual(status.HTTP_200_OK, response.status_code)

    def test_owner_can_download_its_shared_file_even_without_permissions(self):
        self._set_file_as_shared_with_fake_permission()
        self.client.force_authenticate(self.owner)
        response = self.client.get('/v1/files/{}/'.format(self.file.uuid))
        self.assertEqual(status.HTTP_200_OK, response.status_code)

    def test_not_owner_cannot_download_shared_file_without_permission(self):
        self._set_file_as_shared_with_fake_permission()
        self.client.force_authenticate(self.not_owner)
        response = self.client.get('/v1/files/{}/'.format(self.file.uuid))
        self.assertEqual(status.HTTP_403_FORBIDDEN, response.status_code)

    def test_not_owner_can_download_shared_file_with_permissions(self):
        self._set_file_as_shared_with_right_permission()
        self.client.force_authenticate(self.not_owner)
        response = self.client.get('/v1/files/{}/'.format(self.file.uuid))
        self.assertEqual(status.HTTP_200_OK, response.status_code)

    def test_not_owner_can_download_shared_file_with_empty_permissions(self):
        self._set_file_as_shared_with_empty_permission()
        self.client.force_authenticate(self.not_owner)
        response = self.client.get('/v1/files/{}/'.format(self.file.uuid))
        self.assertEqual(status.HTTP_200_OK, response.status_code)

    def test_superuser_can_download_shared_file_even_without_permissions(self):
        self._set_file_as_shared_with_right_permission()
        self.client.force_authenticate(self.superuser)
        response = self.client.get('/v1/files/{}/'.format(self.file.uuid))
        self.assertEqual(status.HTTP_200_OK, response.status_code)



# Generated by Django 3.0.4 on 2020-03-19 11:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangorestfilemanager', '0004_auto_20200318_1806'),
    ]

    operations = [
        migrations.AlterField(
            model_name='file',
            name='status',
            field=models.CharField(max_length=3, verbose_name='Status'),
        ),
    ]



from django.db import migrations, models
import djangorestfilemanager.models


class Migration(migrations.Migration):

    dependencies = [
        ('djangorestfilemanager', '0006_auto_20200429_1220'),
    ]

    operations = [
        migrations.AlterField(
            model_name='file',
            name='name',
            field=models.CharField(max_length=255, verbose_name='Name'),
        ),
    ]



from django.conf import settings

VERSION = '1.1.7'

REST_FILE_MANAGER = getattr(settings, 'REST_FILE_MANAGER', {})
UPLOADS_DIR = REST_FILE_MANAGER.get('UPLOADS_DIR', 'files/')
UPLOAD = 'UPL'
CHOICES = ((UPLOAD, 'Upload'),)
STATUS_CHOICES = REST_FILE_MANAGER.get('STATUS_CHOICES', CHOICES)
DEFAULT_CHOICE = REST_FILE_MANAGER.get('DEFAULT_CHOICE', UPLOAD)
DEFAULT_SHAREABLE = REST_FILE_MANAGER.get('DEFAULT_SHAREABLE', True)



# Generated by Django 3.0.3 on 2020-03-11 09:22

from django.db import migrations, models
import djangorestfilemanager.models


class Migration(migrations.Migration):

    dependencies = [
        ('djangorestfilemanager', '0002_auto_20200302_0947'),
    ]

    operations = [
        migrations.AddField(
            model_name='file',
            name='status',
            field=models.CharField(choices=[('PNP', 'Pending'), ('DIG', 'Doing'), ('UPL', 'Upload'), ('ERR', 'Error')], default='UPL', max_length=3, verbose_name='Status'),
        ),
        migrations.AlterField(
            model_name='file',
            name='file',
            field=models.FileField(null=True, storage=djangorestfilemanager.models.RestFileManagerStorage(location='media'), upload_to=djangorestfilemanager.models.set_storage_file_dir, verbose_name='File'),
        ),
    ]
